from .exceptions import Error, ClientError, InvalidResponseError, NotFoundError, InvalidParameterError

"""
aspxstats.
Python library for retrieving stats of Battlefield 2 and Battlefield 2142 players.
"""

__version__ = '0.0.1'
__author__ = 'cetteup'
__credits__ = 'wilson212'
__all__ = ['bf2', 'Error', 'ClientError', 'InvalidResponseError', 'NotFoundError', 'InvalidParameterError']
